;;; -*- Mode: Lisp; Syntax: ANSI-Common-Lisp; Base: 10; Package: bosco -*-
;;;; ***********************************************************************
;;;; FILE IDENTIFICATION
;;;;
;;;; Name:          main.lisp
;;;; Project:       Bosco - Application framework for OSX and OpenMCL
;;;; Purpose:       Bosco 0.6 main program
;;;; Programmer:    mikel evins
;;;; Date Started:  05/12/2004
;;;;
;;;; $Id: main.lisp,v 1.4 2007/02/05 01:04:44 mikel Exp $
;;;; ***********************************************************************

;;; ----------------------------------------------------------------------
;;; application delegate stuff
;;; ----------------------------------------------------------------------

(in-package "CCL")

;;; from ccl examples/cocoa-application.lisp
(defun reset-ccl-directory ()
  (or (getenv "CCL_DEFAULT_DIRECTORY")
      (with-autorelease-pool
          (let* ((bundle (send (@class ns-bundle) 'main-bundle))
                 (ccl-dir (unless (%null-ptr-p bundle)
                            (send bundle :object-for-info-dictionary-key
                                  #@"CCLDefaultDirectory")))
                 (bundle-path (unless (%null-ptr-p bundle)
                                (send bundle 'bundle-path))))
            (when (and ccl-dir (not (%null-ptr-p ccl-dir))
                       bundle-path (not (%null-ptr-p bundle-path)))
              (let* ((bundle-string (lisp-string-from-nsstring bundle-path))
                     (ccl-string (lisp-string-from-nsstring ccl-dir))
                     (bundle-len (length bundle-string)))
                (if (and (> bundle-len 0)
                         (not (eql (schar bundle-string (1- bundle-len)) #\/)))
                  (setq bundle-string (concatenate 'string bundle-string "/")))
                (let* ((default-dir (native-translated-namestring
                                     (merge-pathnames ccl-string bundle-string))))
                  (setenv "CCL_DEFAULT_DIRECTORY" default-dir t)
                  (init-logical-directories))))))))

;;; The application delegate gets notified of state changes in the
;;; application object.
(defclass lisp-application-delegate (ns:ns-object)
    ()
  (:metaclass ns:+ns-object))


(define-objc-method ((:void :application-will-finish-launching (:id notification))
		     lisp-application-delegate)
  (declare (ignore notification))
  (initialize-user-interface))

(define-objc-method ((:void :application-will-terminate (:id notification))
                     lisp-application-delegate)
  (declare (ignore notification))
  ;; UI has decided to quit; terminate other lisp threads.
  (prepare-to-quit))

(define-objc-method ((:void :new-listener sender) lisp-application-delegate)
  (declare (ignore sender))
  (send (send (@class ns-document-controller) 'shared-document-controller)
	:open-untitled-document-of-type #@"Listener" :display t))

(defvar *cocoa-application-finished-launching* (make-semaphore)
  "Semaphore that's signaled when the application's finished launching ...")

(define-objc-method ((:void :application-did-finish-launching notification)
		     lisp-application-delegate)
  (declare (ignore notification))
  (signal-semaphore *cocoa-application-finished-launching*))

(define-objc-method ((:<BOOL> :application-open-untitled-file app)
		     lisp-application-delegate)
  (when (zerop *cocoa-listener-count*)
    (send self :new-listener app)
    t))


(defmethod ui-object-do-operation ((o ns:ns-application)
                                   operation
                                   &rest args)
  (declare (ignore operation args))
  ;; Do nothing.  Would it be better to warn and/or log this ?
  )

(defmethod ui-object-do-operation ((o ns:ns-application)
                                   (operation (eql :note-current-package))
                                   &rest args)
  (ui-object-note-package o (car args)))

(defmethod ui-object-do-operation ((o ns:ns-application)
                                   (operation (eql :eval-selection))
                                   &rest args)
  (ui-object-eval-selection o (car args)))

(defmethod ui-object-do-operation ((o ns:ns-application)
                                   (operation (eql :enter-backtrace-context))
                                   &rest args)
  (ui-object-enter-backtrace-context o (car args)))

(defmethod ui-object-do-operation ((o ns:ns-application)
                                   (operation (eql :exit-backtrace-context))
                                   &rest args)
  (ui-object-exit-backtrace-context o (car args)))

;;; ----------------------------------------------------------------------
;;; the application main function
;;; ----------------------------------------------------------------------

(in-package "BOSCO")

;;; the toplevel-function runs the Cocoa NSApp object

(defmethod toplevel-function ((app bosco-cocoa-application) init-file)
  (declare (ignore init-file))
  (ccl::reset-ccl-directory)
  (with-autorelease-pool
    (or *NSApp* (setq *NSApp* (init-cocoa-application)))
    (run-event-loop)))


