;;;; -*- Mode: Lisp; Syntax: ANSI-Common-Lisp; Base: 10; Package: bosco -*-
;;;; ***********************************************************************
;;;; FILE IDENTIFICATION
;;;;
;;;; Name:          cocoa.lisp
;;;; Project:       Bosco - Application framework for OSX and OpenMCL
;;;; Purpose:       Main application for Cocoa builds
;;;; Programmer:    mikel evins
;;;; Date Started:  05/12/2004
;;;;
;;;; $Id: cocoa.lisp,v 1.1.1.1 2006/12/27 23:33:36 mikel Exp $
;;;; ***********************************************************************

;;; ABOUT

;;; The purpose of application.lisp is to provide a base class
;;; for Bosco applications so that clients of the Bosco framework
;;; can extend annd customize applications to create their own
;;; Bosco-based projects.

(in-package "CCL")


;;; ------------------------------------------------------------
;;; load Cocoa support

(eval-when (:compile-toplevel :load-toplevel :execute)
  (use-interface-dir :cocoa)
  (open-shared-library "/System/Library/Frameworks/Cocoa.framework/Cocoa")
  (require "OBJC-SUPPORT"))

;;; ------------------------------------------------------------
;;; handy Cocoa-related globals

(defparameter *NSApp* nil)
(defparameter *main-bundle-path* nil)

;;; ------------------------------------------------------------
;;; logging utility

(defun nslog (c)
  (let* ((rep (format nil "~a" c)))
    (ccl::with-cstrs ((str rep))
      (ccl::with-nsstr (nsstr str (length rep))
        (#_NSLog #@"Logging: %@" :address nsstr)))))

;;; ------------------------------------------------------------
;;; setup Cocoa app object to run

(defun init-cocoa-application ()
  (with-autorelease-pool
      (let* ((appclass (#_NSClassFromString (%make-nsstring "NSApplication")))
             (app (send appclass 'shared-application))
             (main-bundle (send (@class ns-bundle) 'main-bundle))
             (main-bundle-path-nsstring (send main-bundle 'bundle-path))
             (main-bundle-path (pathname 
                                (lisp-string-from-nsstring main-bundle-path-nsstring)))
             (info-dict (send main-bundle 'info-dictionary))
             (main-nib-name (send info-dict :object-for-key #@"NSMainNibFile")))
        ;; ----------------------------------------
        ;; init important paths
        (setq *main-bundle-path* main-bundle-path)
        ;; ----------------------------------------
        ;; load the application nib
        (send (@class ns-bundle)
              :load-nib-named main-nib-name
              :owner app)
        app)))

;;; ------------------------------------------------------------
;;; run the Cocoa app's event loop

(defun run-event-loop ()
  (%set-toplevel nil)
  (let* ((app *NSApp*))
    (loop
     (handler-case (send (the ns-application app) 'run)
       (error (c) (nslog c)))
     (unless (send app 'is-running)
       (return)))
    ;; This is a little funky (OK, it's a -lot- funky.) The
    ;; -[NSApplication _deallocHardCore:] method wants an autorelease
    ;; pool to be established when it's called, but one of the things
    ;; that it does is to release all autorelease pools.  So, we create
    ;; one, but don't worry about freeing it ...
    (create-autorelease-pool)
    (objc-message-send app "_deallocHardCore:" :<BOOL> #$YES :void)))


